import json
from datetime import date, timedelta
from enum import Enum, IntFlag
import logging

from service_days import __version__

LOG_FORMAT: str = f"v{__version__} | %(levelname)s | %(asctime)s | %(funcName)s | %(message)s"
LOG_LEVEL: str = "DEBUG"
logging.basicConfig(level=LOG_LEVEL, format=LOG_FORMAT)
logger = logging.getLogger(__name__)


# Make some Constants
class Day(IntFlag):
    MON = 0
    TUE = 1
    WED = 2
    THU = 3
    FRI = 4
    SAT = 5
    SUN = 6


def service_day_add(start_date: date, work_days_to_add: int,
                    week_schedule=(Day.MON, Day.TUE, Day.WED, Day.THU, Day.FRI)) -> date:
    """
    Everything prior to "next service" day is considered part of the current service day.  Given a M-W Schedule, and
    given the current date is a Saturday or Sunday, the date math is based on concept the current service day is
    Friday.

    :param start_date:
    :param work_days_to_add:
    :param week_schedule:
    :return:
    """
    logger.debug(f"ENTERING: service_day_add: adding {work_days_to_add} days to {start_date} with "
                 f"schedule of {week_schedule}")

    weeks, days = divmod(work_days_to_add, len(week_schedule))
    logger.debug(f"Adjusting for weeks: {weeks}, days: {days}")

    cur_svc_date = start_date  # start with assumption that current date is a service day
    min_sched_day = min([d.value for d in week_schedule])
    max_sched_day = max([d.value for d in week_schedule])

    cur_day = Day(start_date.weekday())
    if cur_day not in week_schedule:
        if cur_day.value < min_sched_day:
            # max sched day, previous week
            cur_svc_date = start_date - timedelta(weeks=1)
            cur_svc_date = cur_svc_date + timedelta((max_sched_day - cur_svc_date.weekday()) % 7)
            logger.debug(f"Adjusting for current day not in schedule: {cur_day} < {min_sched_day}")
            logger.debug(f"New base date date: {cur_svc_date}")

        elif min_sched_day < cur_day.value < max_sched_day:
            # go backwards by day,until we hit previous sched day
            while cur_day not in week_schedule:
                cur_svc_date -= timedelta(days=1)
                cur_day = Day(cur_svc_date.weekday())

        elif max_sched_day < cur_day.value:
            # go backwards by day,until we hit previous sched day
            while cur_day not in week_schedule:
                cur_svc_date -= timedelta(days=1)
                cur_day = Day(cur_svc_date.weekday())
        else:
            raise ValueError(f"Unexpected condition: {cur_day}")

    new_date = cur_svc_date + timedelta(weeks=weeks)
    if len(week_schedule) > 1:
        if days == 0:
            # if weeks > len(week_schedule):
            #     new_date -= timedelta(new_date.weekday() - len(week_schedule) + 1)
            # elif new_date.weekday() > len(week_schedule) - 1:
            #     new_date -= timedelta(new_date.weekday() - len(week_schedule) + 1)
            # Make sure new_date is in the week_schedule
            t_day = Day(new_date.weekday())
            while t_day not in week_schedule:
                # If result is not one of the days in the week schedule, increment by 1 day
                new_date += timedelta(days=1)
                t_day = Day(new_date.weekday())

        else:
            for _ in range(days):
                # increment # of days passed in
                new_date += timedelta(days=1)
                t_day = Day(new_date.weekday())

                while t_day not in week_schedule:
                    # If result is not one of the days in the week schedule, increment by 1 day
                    new_date += timedelta(days=1)
                    t_day = Day(new_date.weekday())

    else:
        business_date = []
        for wd in week_schedule:
            if wd >= (7 - start_date.weekday()):
                next_day = (wd - (7 - start_date.weekday()) + 1) + (7 * (work_days_to_add - 1))
            else:
                next_day = (wd + start_date.weekday() + 1) + (7 * (work_days_to_add - 1))
            business_date.append(next_day)
        start_date += timedelta(min(business_date))
        new_date = start_date
    return new_date


def map_schedule_txt_list_to_day_num_list(process_sched_day_list, in_place=False):
    if isinstance(process_sched_day_list, str):
        str_to_list = json.loads(process_sched_day_list)
        process_sched_day_list = [elem.strip() for elem in str_to_list]

    if in_place:
        work_list = process_sched_day_list
    elif process_sched_day_list is None:
        return range(7)

    else:
        work_list = list(range(len(process_sched_day_list)))

    for index, item in enumerate(process_sched_day_list):

        _extracted_from_map_schedule_txt_list_to_day_num_list_16(
            item, work_list, index
        )
    return work_list


def _extracted_from_map_schedule_txt_list_to_day_num_list_16(item, work_list, index):
    if item.strip().casefold() == "Mon".casefold():
        work_list[index] = Day.MON.value

    if item.strip().casefold() == "Tue".casefold():
        work_list[index] = Day.TUE.value

    if item.strip().casefold() == "Wed".casefold():
        work_list[index] = Day.WED.value

    if item.strip().casefold() == "Thu".casefold():
        work_list[index] = Day.THU.value

    if item.strip().casefold() == "Fri".casefold():
        work_list[index] = Day.FRI.value

    if item.strip().casefold() == "Sat".casefold():
        work_list[index] = Day.SAT.value

    if item.strip().casefold() == "Sun".casefold():
        work_list[index] = Day.SUN.value



from unittest import TestCase
from service_days.servicedays import service_day_add, Day

from datetime import date

from hamcrest import *


class Test(TestCase):

    def test_service_day_add_simple_add_to_default_week(self):
        # Default Schedule (Mon-Fri)
        # Monday -> Tuesday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, 1), equal_to(date(2023, 3, 7)))

        # Tuesday -> Wednesday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, 1), equal_to(date(2023, 3, 8)))

        # Wednesday -> Thursday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, 1), equal_to(date(2023, 3, 9)))

        # Thursday -> Friday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, 1), equal_to(date(2023, 3, 10)))

        # Friday -> Monday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, 1), equal_to(date(2023, 3, 13)))

        # Saturday -> Monday
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, 1), equal_to(date(2023, 3, 13)))

        # Sunday -> Monday
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, 1), equal_to(date(2023, 3, 13)))

        # --- ADD 2 ---
        # Default Schedule (Mon-Fri)
        add_days = 2
        # Monday -> Wednesday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 8)))

        # Tuesday -> Thursday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 9)))

        # Wednesday -> Friday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 10)))

        # Thursday -> Monday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 13)))

        # Friday -> Tuesday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 14)))

        # Saturday -> Tu
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 14)))

        # Sunday -> Tuesday
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 14)))

        # --- ADD 3 ---
        # Default Schedule (Mon-Fri)
        add_days = 3
        # Monday -> Thursday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 9)))

        # Tuesday -> Friday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 10)))

        # Wednesday -> Monday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 13)))

        # Thursday -> Tuesday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 14)))

        # Friday -> Wednesday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 15)))

        # Saturday -> Wednesday
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 15)))

        # Sunday -> Wednesday
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 15)))

        #
        # Default Schedule (Mon-Fri)
        add_days = 4
        # Monday -> Friday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 10)))

        # Tuesday -> Monday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 13)))

        # Wednesday -> Tuesday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 14)))

        # Thursday -> Wednesday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 15)))

        # Friday -> Thursday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 16)))

        # Saturday -> Thursday
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 16)))

        # Sunday -> Thursday
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 16)))

        #
        # Default Schedule (Mon-Fri)
        add_days = 5
        # Monday -> Monday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 13)))

        # Tuesday -> Tuesday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 14)))

        # Wednesday -> Wednesday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 15)))

        # Thursday -> Thursday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 16)))

        # Friday -> Friday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 17)))

        # Saturday -> Friday
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 17)))

        # Sunday -> Friday
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 17)))

        # --- ADD 6 ---
        # Default Schedule (Mon-Fri)
        add_days = 6

        # Monday -> Next Tuesday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 14)))

        # Tuesday -> Next Wednesday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 15)))

        # Wednesday -> Next Thursday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 16)))

        # Thursday -> Next Friday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 17)))

        # Friday -> Monday After Next
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 20)))

        # Saturday -> Monday After Next
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 20)))

        # Sunday -> Monday After Next
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 20)))

        # --- ADD 7 ---
        # Default Schedule (Mon-Fri)
        add_days = 7

        # Monday -> Next Wednesday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 15)))

        # Tuesday -> Next Thursday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 16)))

        # Wednesday -> Next Friday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 17)))

        # Thursday -> Monday After Next
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 20)))

        # Friday -> Tuesday After Next
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 21)))

        # Saturday -> Tuesday After Next
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 21)))

        # Sunday -> Monday After Next
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 21)))

        # --- ADD 8 ---
        # Default Schedule (Mon-Fri)
        add_days = 8

        # Monday -> Next Thursday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 16)))

        # Tuesday -> Next Friday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 17)))

        # Wednesday -> Monday After Next
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 20)))

        # Thursday -> Tuesday After Next
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 21)))

        # Friday -> Wednesday After Next
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 22)))

        # Saturday -> Wednesday After Next
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 22)))

        # Sunday -> Wednesday After Next
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days), equal_to(date(2023, 3, 22)))

    def test_service_day_add_to_fri_sat_sun(self):
        schedule = (Day.FRI, Day.SAT, Day.SUN)
        # Monday -> Friday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, 1, schedule), equal_to(date(2023, 3, 10)))

        # Tuesday -> Friday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, 1, schedule), equal_to(date(2023, 3, 10)))

        # Wednesday -> Friday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, 1, schedule), equal_to(date(2023, 3, 10)))

        # Thursday -> Friday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, 1, schedule), equal_to(date(2023, 3, 10)))

        # Friday -> Saturday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, 1, schedule), equal_to(date(2023, 3, 11)))

        # Saturday -> Sunday
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, 1, schedule), equal_to(date(2023, 3, 12)))

        # Sunday -> Friday
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, 1, schedule), equal_to(date(2023, 3, 17)))

        # --- ADD 2 ---
        add_days = 2
        # Monday -> Saturday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 11)))

        # Tuesday -> Saturday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 11)))

        # Wednesday -> Saturday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 11)))

        # Thursday -> Saturday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 11)))

        # Friday -> Sunday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 12)))

        # Saturday -> Friday
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 17)))

        # Sunday -> Sunday
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 18)))

        # --- ADD 3 ---

        add_days = 3
        # Monday -> Sunday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 12)))

        # Tuesday -> Sunday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 12)))

        # Wednesday -> Sunday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 12)))

        # Thursday -> Sunday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 12)))

        # Friday -> Next Friday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 17)))

        # Saturday -> Next Saturday
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 18)))

        # Sunday -> Next Sunday
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 19)))

        # --- ADD 4 ---
        add_days = 4
        # Monday -> Next Friday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 17)))

        # Tuesday -> Next Friday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 17)))

        # Wednesday -> Next Friday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 17)))

        # Thursday -> Next Friday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 17)))

        # Friday -> Next Saturday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 18)))

        # Saturday -> Next Sunday
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 19)))

        # Sunday -> Friday after next
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 24)))

        # --- ADD 5 ---
        add_days = 5
        # Monday -> Next Saturday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 18)))

        # Tuesday -> Next Saturday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 18)))

        # Wednesday -> Next Saturday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 18)))

        # Thursday -> Next Saturday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 18)))

        # Friday -> Next Sunday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 19)))

        # Saturday -> Friday After Next
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 24)))

        # Sunday -> Saturday After Next
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 25)))

        # --- ADD 6 ---
        add_days = 6

        # Monday -> Sunday after next
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 19)))

        # Tuesday -> Sunday after next
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 19)))

        # Wednesday -> Sunday after next
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 19)))

        # Thursday -> Sunday after next
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 19)))

        # Friday -> Friday After Next
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 24)))

        # Saturday -> Saturday After Next
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 25)))

        # Sunday -> Sunday After Next
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 26)))

        # --- ADD 7 ---
        add_days = 7

        # Monday -> Friday after Next
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 24)))

        # Tuesday -> Friday after next
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 24)))

        # Wednesday -> Friday after next
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 24)))

        # Thursday -> Friday after next
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 24)))

        # Friday -> Saturday After Next
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 25)))

        # Saturday -> Sunday after next
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 26)))

        # Sunday -> Three Fridays out
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 31)))

        # --- ADD 8 ---
        # Default Schedule (Mon-Fri)
        add_days = 8

        # Monday -> Third Saturday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 25)))

        # Tuesday -> Third Saturday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 25)))

        # Wednesday -> Third Saturday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 25)))

        # Thursday -> Third Saturday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 25)))

        # Friday -> Third Sunday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 26)))

        # Saturday -> Third Friday
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 31)))

        # Sunday -> Third Saturday
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 4, 1)))

    def test_service_day_add_to_mon_wed_sat(self):
        schedule = (Day.MON, Day.WED, Day.SAT)
        add_days = 1
        # Monday -> Wednesday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 8)))

        # Tuesday -> Wednesday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 8)))

        # Wednesday -> Saturday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 11)))

        # Thursday -> Saturday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 11)))

        # Friday -> Saturday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 11)))

        # Saturday -> Monday
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 13)))

        # Sunday -> Monday
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 13)))

        # --- ADD 2 ---
        add_days = 2
        # Monday -> Saturday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 11)))

        # Tuesday -> Saturday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 11)))

        # Wednesday -> Monday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 13)))

        # Thursday -> Monday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 13)))

        # Friday -> Monday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 13)))

        # Saturday -> Wednesday
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 15)))

        # Sunday -> Wednesday
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 15)))

        # --- ADD 3 ---

        add_days = 3
        # Monday -> Monday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 13)))

        # Tuesday -> Monday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 13)))

        # Wednesday -> Wednesday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 15)))

        # Thursday -> Wednesday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 15)))

        # Friday -> Wednesday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 15)))

        # Saturday ->  Saturday
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 18)))

        # Sunday -> Saturday
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 18)))

        # --- ADD 4 ---
        add_days = 4
        # Monday -> Next Wednesday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 15)))

        # Tuesday -> Next Wednesday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 15)))

        # Wednesday -> Next Saturday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 18)))

        # Thursday -> Next Saturday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 18)))

        # Friday -> Next Saturday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 18)))

        # Saturday -> Next Monday
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 20)))

        # Sunday -> Next Monday
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 20)))

        # --- ADD 5 ---
        add_days = 5
        # Monday -> Next Saturday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 18)))

        # Tuesday -> Next Saturday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 18)))

        # Wednesday -> Next Monday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 20)))

        # Thursday -> Next Monday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 20)))

        # Friday -> Next Monday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 20)))

        # Saturday -> Next Wednesday
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 22)))

        # Sunday -> Next Wednesday
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 22)))

        # --- ADD 6 ---
        add_days = 6

        # Monday -> Monday after next
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 20)))

        # Tuesday -> Monday after next
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 20)))

        # Wednesday -> Wednesday after next
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 22)))

        # Thursday -> Wednesday after next
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 22)))

        # Friday -> Wednesday After Next
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 22)))

        # Saturday -> Saturday After Next
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 25)))

        # Sunday -> Saturday After Next
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 25)))

        # --- ADD 7 ---
        add_days = 7

        # Monday -> Wednesday after Next
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 22)))

        # Tuesday -> Wednesday after next
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 22)))

        # Wednesday -> Saturday after next
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 25)))

        # Thursday -> Saturday after next
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 25)))

        # Friday -> Saturday After Next
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 25)))

        # Saturday -> Monday after next
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 27)))

        # Sunday -> Monday after next
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 27)))

        # --- ADD 8 ---
        # Default Schedule (Mon-Fri)
        add_days = 8

        # Monday -> Third Saturday
        t_date = date(2023, 3, 6)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 25)))

        # Tuesday -> Third Saturday
        t_date = date(2023, 3, 7)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 25)))

        # Wednesday -> Third Monday
        t_date = date(2023, 3, 8)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 27)))

        # Thursday -> Third Monday
        t_date = date(2023, 3, 9)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 27)))

        # Friday -> Third Monday
        t_date = date(2023, 3, 10)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 27)))

        # Saturday -> Third Wednesday
        t_date = date(2023, 3, 11)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 29)))

        # Sunday -> Third Wednesday
        t_date = date(2023, 3, 12)
        assert_that(service_day_add(t_date, add_days, schedule), equal_to(date(2023, 3, 29)))
